@@ -62,13 +62,7 @@ static inline int get_byte(LZOContext *c) {
 static inline int get_len(LZOContext *c, int x, int mask) {
     int cnt = x & mask;
     if (!cnt) {
-        while (!(x = get_byte(c))) {
-            if (cnt >= INT_MAX - 1000) {
-                c->error |= AV_LZO_ERROR;
-                break;
-            }
-            cnt += 255;
-        }
+        while (!(x = get_byte(c))) cnt += 255;
         cnt += mask + x;
     }
     return cnt;
